@@ -1,15 +1,16 @@
+import errno
 import time
 from contextlib import contextmanager
 from enum import Enum
+from os import PathLike
 from pathlib import Path
 from tempfile import gettempdir
-from typing import Any, Callable, Generator, Union, Optional, TYPE_CHECKING
+from typing import Generator, Union, Optional
 
-if TYPE_CHECKING:
-    from os import PathLike
+from . import utils
 
 __author__ = "Linus Groh"
-__version__ = "2.3.0"
+__version__ = "2.4.0"
 __all__ = ["Backlight", "BoardType"]
 
 
@@ -46,14 +47,12 @@ class Backlight:
     def __init__(
         self,
         backlight_sysfs_path: Optional[Union[str, "PathLike[str]"]] = None,
-        board_type: BoardType = BoardType.RASPBERRY_PI,
+        board_type: BoardType = utils.detect_board_type() or BoardType.RASPBERRY_PI,
     ):
         """Set ``backlight_sysfs_path`` to ``":emulator:"`` to use with rpi-backlight-emulator."""
         if not isinstance(board_type, BoardType):
             raise TypeError(
-                "board_type must be a member of the BoardType enum, got {0}".format(
-                    type(board_type)
-                )
+                f"board_type must be a member of the BoardType enum, got {type(board_type)}"
             )
 
         if not backlight_sysfs_path:
@@ -61,11 +60,13 @@ class Backlight:
         elif backlight_sysfs_path == _EMULATOR_MAGIC_STRING:
             if not _EMULATOR_SYSFS_TMP_FILE_PATH.exists():
                 raise RuntimeError(
-                    "Emulator seems to be not running, {0} not found".format(
-                        _EMULATOR_SYSFS_TMP_FILE_PATH
-                    )
+                    f"Emulator seems to be not running, {_EMULATOR_SYSFS_TMP_FILE_PATH} not found"
                 )
             backlight_sysfs_path = _EMULATOR_SYSFS_TMP_FILE_PATH.read_text()
+            # The emulator only knows about Raspberry Pi sysfs files
+            # (brightness, bl_power), ignore board_type
+            board_type = BoardType.RASPBERRY_PI
+
         self._backlight_sysfs_path = Path(backlight_sysfs_path)
         self._board_type = board_type
         self._fade_duration = 0.0  # in seconds
@@ -86,7 +87,7 @@ class Backlight:
             # Try again
             return self._get_value(name)
         except (OSError, IOError) as e:
-            if e.errno == 13:
+            if e.errno == errno.EPERM:
                 _permission_denied()
             raise e
 
@@ -94,15 +95,15 @@ class Backlight:
         try:
             (self._backlight_sysfs_path / name).write_text(str(value))
         except (OSError, IOError) as e:
-            if e.errno == 13:
+            if e.errno == errno.EPERM:
                 _permission_denied()
             raise e
 
     def _normalize_brightness(self, value: float) -> int:
-        return int(round(value / self._max_brightness * 100))
+        return max(min(100, int(round(value / self._max_brightness * 100))), 0)
 
     def _denormalize_brightness(self, value: float) -> int:
-        return int(round(value * self._max_brightness / 100))
+        return max(min(255, int(round(value * self._max_brightness / 100))), 0)
 
     @contextmanager
     def fade(self, duration: float) -> Generator:
@@ -141,9 +142,9 @@ class Backlight:
         """Set the fade duration."""
         # isinstance(True, int) is True, so additional check for bool.
         if not isinstance(duration, (int, float)) or isinstance(duration, bool):
-            raise TypeError("value must be a number, got {0}".format(type(duration)))
+            raise TypeError(f"value must be a number, got {type(duration)}")
         if duration < 0:
-            raise ValueError("value must be >= 0, got {0}".format(duration))
+            raise ValueError(f"value must be >= 0, got {duration}")
         self._fade_duration = duration
 
     @property
@@ -174,14 +175,18 @@ class Backlight:
         """Set the display brightness."""
         # isinstance(True, int) is True, so additional check for bool.
         if not isinstance(value, (int, float)) or isinstance(value, bool):
-            raise TypeError("value must be a number, got {0}".format(type(value)))
+            raise TypeError(f"value must be a number, got {type(value)}")
         if value < 0 or value > 100:
-            raise ValueError("value must be in range 0-100, got {0}".format(value))
+            raise ValueError(f"value must be in range 0-100, got {value}")
         if self.fade_duration > 0:
             current_value = self.brightness
             step = 1 if current_value < value else -1
             diff = abs(value - current_value)
-            while current_value != value:
+            while (
+                0.0 <= current_value
+                and current_value != value
+                and current_value <= 100.0
+            ):
                 current_value += step
                 if self._board_type == BoardType.RASPBERRY_PI:
                     self._set_value(
@@ -236,7 +241,7 @@ class Backlight:
     def power(self, on: bool) -> None:
         """Set the display power on or off."""
         if not isinstance(on, bool):
-            raise TypeError("value must be a bool, got {0}".format(type(on)))
+            raise TypeError(f"value must be a bool, got {type(on)}")
         if self._board_type == BoardType.RASPBERRY_PI:
             # 0 is on, 1 is off
             self._set_value("bl_power", int(not on))
